@@ -48,7 +48,8 @@ from sphinx.util.tags import Tags
 
 if False:
     # For type annotation
-    from typing import Any, Callable, Dict, IO, Iterable, Iterator, List, Tuple, Type, Union  # NOQA
+    from typing import Any, Callable, Dict, IO, Iterable, Iterator, List, Tuple, Union  # NOQA
+    from typing import Type  # for python3.5.1
     from docutils import nodes  # NOQA
     from docutils.parsers import Parser  # NOQA
     from docutils.transforms import Transform  # NOQA
@@ -1054,8 +1055,8 @@ class Sphinx:
         else:
             lexer_classes[alias] = lexer
 
-    def add_autodocumenter(self, cls):
-        # type: (Any) -> None
+    def add_autodocumenter(self, cls, override=False):
+        # type: (Any, bool) -> None
         """Register a new documenter class for the autodoc extension.
 
         Add *cls* as a new documenter class for the :mod:`sphinx.ext.autodoc`
@@ -1067,11 +1068,13 @@ class Sphinx:
         .. todo:: Add real docs for Documenter and subclassing
 
         .. versionadded:: 0.6
+        .. versionchanged:: 2.2
+           Add *override* keyword.
         """
         logger.debug('[app] adding autodocumenter: %r', cls)
         from sphinx.ext.autodoc.directive import AutodocDirective
         self.registry.add_documenter(cls.objtype, cls)
-        self.add_directive('auto' + cls.objtype, AutodocDirective)
+        self.add_directive('auto' + cls.objtype, AutodocDirective, override=override)
 
     def add_autodoc_attrgetter(self, typ, getter):
         # type: (Type, Callable[[Any, str, Any], Any]) -> None
