@@ -9,6 +9,7 @@
 """
 
 import warnings
+from typing import Any, Dict
 
 from sphinxcontrib.applehelp import (
     AppleHelpCodeSigningFailed,
@@ -16,13 +17,9 @@ from sphinxcontrib.applehelp import (
     AppleHelpBuilder,
 )
 
+from sphinx.application import Sphinx
 from sphinx.deprecation import RemovedInSphinx40Warning, deprecated_alias
 
-if False:
-    # For type annotation
-    from typing import Any, Dict  # NOQA
-    from sphinx.application import Sphinx  # NOQA
-
 
 deprecated_alias('sphinx.builders.applehelp',
                  {
@@ -33,8 +30,7 @@ deprecated_alias('sphinx.builders.applehelp',
                  RemovedInSphinx40Warning)
 
 
-def setup(app):
-    # type: (Sphinx) -> Dict[str, Any]
+def setup(app: Sphinx) -> Dict[str, Any]:
     warnings.warn('sphinx.builders.applehelp has been moved to sphinxcontrib-applehelp.',
                   RemovedInSphinx40Warning)
     app.setup_extension('sphinxcontrib.applehelp')
