@@ -14,7 +14,8 @@ from importlib import import_module
 
 if False:
     # For type annotation
-    from typing import Any, Dict, Type  # NOQA
+    from typing import Any, Dict  # NOQA
+    from typing import Type  # for python3.5.1
 
 
 class RemovedInSphinx30Warning(DeprecationWarning):
@@ -30,10 +31,7 @@ RemovedInNextVersionWarning = RemovedInSphinx30Warning
 
 def deprecated_alias(modname, objects, warning):
     # type: (str, Dict, Type[Warning]) -> None
-    module = sys.modules.get(modname)
-    if module is None:
-        module = import_module(modname)
-
+    module = import_module(modname)
     sys.modules[modname] = _ModuleWrapper(module, modname, objects, warning)  # type: ignore
 
 
@@ -48,8 +46,8 @@ class _ModuleWrapper:
     def __getattr__(self, name):
         # type: (str) -> Any
         if name in self._objects:
-            warnings.warn("%s.%s is now deprecated. Please refer CHANGES to grasp "
-                          "the changes of Sphinx API." % (self._modname, name),
+            warnings.warn("%s.%s is deprecated. Check CHANGES for Sphinx "
+                          "API modifications." % (self._modname, name),
                           self._warning, stacklevel=3)
             return self._objects[name]
 
