@@ -15,16 +15,27 @@ import re
 import sys
 import typing
 import warnings
-from functools import partial
+from functools import partial, partialmethod
+from inspect import (  # NOQA
+    isclass, ismethod, ismethoddescriptor, isroutine
+)
 from io import StringIO
+from typing import Any, Callable, Mapping, List, Tuple
 
 from sphinx.deprecation import RemovedInSphinx30Warning
 from sphinx.util import logging
 from sphinx.util.typing import NoneType
 
-if False:
-    # For type annotation
-    from typing import Any, Callable, Mapping, List, Tuple, Type  # NOQA
+if sys.version_info > (3, 7):
+    from types import (
+        ClassMethodDescriptorType,
+        MethodDescriptorType,
+        WrapperDescriptorType
+    )
+else:
+    ClassMethodDescriptorType = type(object.__init__)
+    MethodDescriptorType = type(str.join)
+    WrapperDescriptorType = type(dict.__dict__['fromkeys'])
 
 logger = logging.getLogger(__name__)
 
@@ -100,26 +111,22 @@ def getargspec(func):
                                kwonlyargs, kwdefaults, annotations)
 
 
-def isenumclass(x):
-    # type: (Type) -> bool
+def isenumclass(x: Any) -> bool:
     """Check if the object is subclass of enum."""
     return inspect.isclass(x) and issubclass(x, enum.Enum)
 
 
-def isenumattribute(x):
-    # type: (Any) -> bool
+def isenumattribute(x: Any) -> bool:
     """Check if the object is attribute of enum."""
     return isinstance(x, enum.Enum)
 
 
-def ispartial(obj):
-    # type: (Any) -> bool
+def ispartial(obj: Any) -> bool:
     """Check if the object is partial."""
-    return isinstance(obj, partial)
+    return isinstance(obj, (partial, partialmethod))
 
 
-def isclassmethod(obj):
-    # type: (Any) -> bool
+def isclassmethod(obj: Any) -> bool:
     """Check if the object is classmethod."""
     if isinstance(obj, classmethod):
         return True
@@ -129,8 +136,7 @@ def isclassmethod(obj):
     return False
 
 
-def isstaticmethod(obj, cls=None, name=None):
-    # type: (Any, Any, str) -> bool
+def isstaticmethod(obj: Any, cls: Any = None, name: str = None) -> bool:
     """Check if the object is staticmethod."""
     if isinstance(obj, staticmethod):
         return True
@@ -149,8 +155,7 @@ def isstaticmethod(obj, cls=None, name=None):
     return False
 
 
-def isdescriptor(x):
-    # type: (Any) -> bool
+def isdescriptor(x: Any) -> bool:
     """Check if the object is some kind of descriptor."""
     for item in '__get__', '__set__', '__delete__':
         if hasattr(safe_getattr(x, item, None), '__call__'):
@@ -158,20 +163,68 @@ def isdescriptor(x):
     return False
 
 
-def isfunction(obj):
-    # type: (Any) -> bool
+def isabstractmethod(obj: Any) -> bool:
+    """Check if the object is an abstractmethod."""
+    return safe_getattr(obj, '__isabstractmethod__', False) is True
+
+
+def isattributedescriptor(obj: Any) -> bool:
+    """Check if the object is an attribute like descriptor."""
+    if inspect.isdatadescriptor(object):
+        # data descriptor is kind of attribute
+        return True
+    elif isdescriptor(obj):
+        # non data descriptor
+        if isfunction(obj) or isbuiltin(obj) or inspect.ismethod(obj):
+            # attribute must not be either function, builtin and method
+            return False
+        elif inspect.isclass(obj):
+            # attribute must not be a class
+            return False
+        elif isinstance(obj, (ClassMethodDescriptorType,
+                              MethodDescriptorType,
+                              WrapperDescriptorType)):
+            # attribute must not be a method descriptor
+            return False
+        elif type(obj).__name__ == "instancemethod":
+            # attribute must not be an instancemethod (C-API)
+            return False
+        else:
+            return True
+    else:
+        return False
+
+
+def isfunction(obj: Any) -> bool:
     """Check if the object is function."""
     return inspect.isfunction(obj) or ispartial(obj) and inspect.isfunction(obj.func)
 
 
-def isbuiltin(obj):
-    # type: (Any) -> bool
+def isbuiltin(obj: Any) -> bool:
     """Check if the object is builtin."""
     return inspect.isbuiltin(obj) or ispartial(obj) and inspect.isbuiltin(obj.func)
 
 
-def safe_getattr(obj, name, *defargs):
-    # type: (Any, str, str) -> object
+def iscoroutinefunction(obj: Any) -> bool:
+    """Check if the object is coroutine-function."""
+    if hasattr(obj, '__code__') and inspect.iscoroutinefunction(obj):
+        # check obj.__code__ because iscoroutinefunction() crashes for custom method-like
+        # objects (see https://github.com/sphinx-doc/sphinx/issues/6605)
+        return True
+    elif (ispartial(obj) and hasattr(obj.func, '__code__') and
+          inspect.iscoroutinefunction(obj.func)):
+        # partialed
+        return True
+    else:
+        return False
+
+
+def isproperty(obj: Any) -> bool:
+    """Check if the object is property."""
+    return isinstance(obj, property)
+
+
+def safe_getattr(obj: Any, name: str, *defargs) -> Any:
     """A getattr() that turns all exceptions into AttributeErrors."""
     try:
         return getattr(obj, name, *defargs)
@@ -193,8 +246,8 @@ def safe_getattr(obj, name, *defargs):
         raise AttributeError(name)
 
 
-def safe_getmembers(object, predicate=None, attr_getter=safe_getattr):
-    # type: (Any, Callable[[str], bool], Callable) -> List[Tuple[str, Any]]
+def safe_getmembers(object: Any, predicate: Callable[[str], bool] = None,
+                    attr_getter: Callable = safe_getattr) -> List[Tuple[str, Any]]:
     """A version of inspect.getmembers() that uses safe_getattr()."""
     results = []  # type: List[Tuple[str, Any]]
     for key in dir(object):
@@ -208,8 +261,7 @@ def safe_getmembers(object, predicate=None, attr_getter=safe_getattr):
     return results
 
 
-def object_description(object):
-    # type: (Any) -> str
+def object_description(object: Any) -> str:
     """A repr() implementation that returns text safe to use in reST context."""
     if isinstance(object, dict):
         try:
@@ -246,8 +298,7 @@ def object_description(object):
     return s.replace('\n', ' ')
 
 
-def is_builtin_class_method(obj, attr_name):
-    # type: (Any, str) -> bool
+def is_builtin_class_method(obj: Any, attr_name: str) -> bool:
     """If attr_name is implemented at builtin class, return True.
 
         >>> is_builtin_class_method(int, '__init__')
@@ -259,9 +310,9 @@ def is_builtin_class_method(obj, attr_name):
     classes = [c for c in inspect.getmro(obj) if attr_name in c.__dict__]
     cls = classes[0] if classes else object
 
-    if not hasattr(builtins, safe_getattr(cls, '__name__', '')):  # type: ignore
+    if not hasattr(builtins, safe_getattr(cls, '__name__', '')):
         return False
-    return getattr(builtins, safe_getattr(cls, '__name__', '')) is cls  # type: ignore
+    return getattr(builtins, safe_getattr(cls, '__name__', '')) is cls
 
 
 class Parameter:
@@ -273,8 +324,8 @@ class Parameter:
     VAR_KEYWORD = 4
     empty = object()
 
-    def __init__(self, name, kind=POSITIONAL_OR_KEYWORD, default=empty):
-        # type: (str, int, Any) -> None
+    def __init__(self, name: str, kind: int = POSITIONAL_OR_KEYWORD,
+                 default: Any = empty) -> None:
         self.name = name
         self.kind = kind
         self.default = default
@@ -289,8 +340,8 @@ class Signature:
     its return annotation.
     """
 
-    def __init__(self, subject, bound_method=False, has_retval=True):
-        # type: (Callable, bool, bool) -> None
+    def __init__(self, subject: Callable, bound_method: bool = False,
+                 has_retval: bool = True) -> None:
         # check subject is not a built-in class (ex. int, str)
         if (isinstance(subject, type) and
                 is_builtin_class_method(subject, "__new__") and
@@ -335,16 +386,14 @@ class Signature:
             self.skip_first_argument = False
 
     @property
-    def parameters(self):
-        # type: () -> Mapping
+    def parameters(self) -> Mapping:
         if self.partialmethod_with_noargs:
             return {}
         else:
             return self.signature.parameters
 
     @property
-    def return_annotation(self):
-        # type: () -> Any
+    def return_annotation(self) -> Any:
         if self.signature:
             if self.has_retval:
                 return self.signature.return_annotation
@@ -353,8 +402,13 @@ class Signature:
         else:
             return None
 
-    def format_args(self):
-        # type: () -> str
+    def format_args(self, show_annotation: bool = True) -> str:
+        def format_param_annotation(param: inspect.Parameter) -> str:
+            if isinstance(param.annotation, str) and param.name in self.annotations:
+                return self.format_annotation(self.annotations[param.name])
+            else:
+                return self.format_annotation(param.annotation)
+
         args = []
         last_kind = None
         for i, param in enumerate(self.parameters.values()):
@@ -375,15 +429,11 @@ class Signature:
                               param.POSITIONAL_OR_KEYWORD,
                               param.KEYWORD_ONLY):
                 arg.write(param.name)
-                if param.annotation is not param.empty:
-                    if isinstance(param.annotation, str) and param.name in self.annotations:
-                        arg.write(': ')
-                        arg.write(self.format_annotation(self.annotations[param.name]))
-                    else:
-                        arg.write(': ')
-                        arg.write(self.format_annotation(param.annotation))
+                if show_annotation and param.annotation is not param.empty:
+                    arg.write(': ')
+                    arg.write(format_param_annotation(param))
                 if param.default is not param.empty:
-                    if param.annotation is param.empty:
+                    if param.annotation is param.empty or show_annotation is False:
                         arg.write('=')
                         arg.write(object_description(param.default))
                     else:
@@ -392,14 +442,20 @@ class Signature:
             elif param.kind == param.VAR_POSITIONAL:
                 arg.write('*')
                 arg.write(param.name)
+                if show_annotation and param.annotation is not param.empty:
+                    arg.write(': ')
+                    arg.write(format_param_annotation(param))
             elif param.kind == param.VAR_KEYWORD:
                 arg.write('**')
                 arg.write(param.name)
+                if show_annotation and param.annotation is not param.empty:
+                    arg.write(': ')
+                    arg.write(format_param_annotation(param))
 
             args.append(arg.getvalue())
             last_kind = param.kind
 
-        if self.return_annotation is inspect.Parameter.empty:
+        if self.return_annotation is inspect.Parameter.empty or show_annotation is False:
             return '(%s)' % ', '.join(args)
         else:
             if 'return' in self.annotations:
@@ -409,8 +465,7 @@ class Signature:
 
             return '(%s) -> %s' % (', '.join(args), annotation)
 
-    def format_annotation(self, annotation):
-        # type: (Any) -> str
+    def format_annotation(self, annotation: Any) -> str:
         """Return formatted representation of a type annotation.
 
         Show qualified names for types and additional details for types from
@@ -436,8 +491,7 @@ class Signature:
         else:
             return self.format_annotation_old(annotation)
 
-    def format_annotation_new(self, annotation):
-        # type: (Any) -> str
+    def format_annotation_new(self, annotation: Any) -> str:
         """format_annotation() for py37+"""
         module = getattr(annotation, '__module__', None)
         if module == 'typing':
@@ -465,14 +519,15 @@ class Signature:
                 args = ', '.join(self.format_annotation(a) for a in annotation.__args__[:-1])
                 returns = self.format_annotation(annotation.__args__[-1])
                 return '%s[[%s], %s]' % (qualname, args, returns)
+            elif annotation._special:
+                return qualname
             else:
                 args = ', '.join(self.format_annotation(a) for a in annotation.__args__)
                 return '%s[%s]' % (qualname, args)
 
         return qualname
 
-    def format_annotation_old(self, annotation):
-        # type: (Any) -> str
+    def format_annotation_old(self, annotation: Any) -> str:
         """format_annotation() for py36 or below"""
         module = getattr(annotation, '__module__', None)
         if module == 'typing':
@@ -491,22 +546,18 @@ class Signature:
         else:
             qualname = repr(annotation)
 
-        if (hasattr(typing, 'TupleMeta') and
-                isinstance(annotation, typing.TupleMeta) and  # type: ignore
-                not hasattr(annotation, '__tuple_params__')):
-            # This is for Python 3.6+, 3.5 case is handled below
+        if (isinstance(annotation, typing.TupleMeta) and
+                not hasattr(annotation, '__tuple_params__')):  # for Python 3.6
             params = annotation.__args__
-            param_str = ', '.join(self.format_annotation(p) for p in params)
-            return '%s[%s]' % (qualname, param_str)
-        elif (hasattr(typing, 'GenericMeta') and  # for py36 or below
-              isinstance(annotation, typing.GenericMeta)):
-            # In Python 3.5.2+, all arguments are stored in __args__,
-            # whereas __parameters__ only contains generic parameters.
-            #
-            # Prior to Python 3.5.2, __args__ is not available, and all
-            # arguments are in __parameters__.
+            if params:
+                param_str = ', '.join(self.format_annotation(p) for p in params)
+                return '%s[%s]' % (qualname, param_str)
+            else:
+                return qualname
+        elif isinstance(annotation, typing.GenericMeta):
             params = None
             if hasattr(annotation, '__args__'):
+                # for Python 3.5.2+
                 if annotation.__args__ is None or len(annotation.__args__) <= 2:  # type: ignore  # NOQA
                     params = annotation.__args__  # type: ignore
                 else:  # typing.Callable
@@ -515,13 +566,14 @@ class Signature:
                     result = self.format_annotation(annotation.__args__[-1])  # type: ignore
                     return '%s[[%s], %s]' % (qualname, args, result)
             elif hasattr(annotation, '__parameters__'):
+                # for Python 3.5.0 and 3.5.1
                 params = annotation.__parameters__  # type: ignore
             if params is not None:
                 param_str = ', '.join(self.format_annotation(p) for p in params)
                 return '%s[%s]' % (qualname, param_str)
-        elif (hasattr(typing, 'UnionMeta') and  # for py35 or below
-              isinstance(annotation, typing.UnionMeta) and  # type: ignore
-              hasattr(annotation, '__union_params__')):
+        elif (hasattr(typing, 'UnionMeta') and
+              isinstance(annotation, typing.UnionMeta) and
+              hasattr(annotation, '__union_params__')):  # for Python 3.5
             params = annotation.__union_params__
             if params is not None:
                 if len(params) == 2 and params[1] is NoneType:  # type: ignore
@@ -529,9 +581,8 @@ class Signature:
                 else:
                     param_str = ', '.join(self.format_annotation(p) for p in params)
                     return '%s[%s]' % (qualname, param_str)
-        elif (hasattr(typing, 'Union') and  # for py36
-              hasattr(annotation, '__origin__') and
-              annotation.__origin__ is typing.Union):
+        elif (hasattr(annotation, '__origin__') and
+              annotation.__origin__ is typing.Union):  # for Python 3.5.2+
             params = annotation.__args__
             if params is not None:
                 if len(params) == 2 and params[1] is NoneType:  # type: ignore
@@ -539,10 +590,9 @@ class Signature:
                 else:
                     param_str = ', '.join(self.format_annotation(p) for p in params)
                     return 'Union[%s]' % param_str
-        elif (hasattr(typing, 'CallableMeta') and  # for py36 or below
-              isinstance(annotation, typing.CallableMeta) and  # type: ignore
+        elif (isinstance(annotation, typing.CallableMeta) and
               getattr(annotation, '__args__', None) is not None and
-              hasattr(annotation, '__result__')):
+              hasattr(annotation, '__result__')):  # for Python 3.5
             # Skipped in the case of plain typing.Callable
             args = annotation.__args__
             if args is None:
@@ -555,10 +605,9 @@ class Signature:
             return '%s[%s, %s]' % (qualname,
                                    args_str,
                                    self.format_annotation(annotation.__result__))
-        elif (hasattr(typing, 'TupleMeta') and  # for py36 or below
-              isinstance(annotation, typing.TupleMeta) and  # type: ignore
+        elif (isinstance(annotation, typing.TupleMeta) and
               hasattr(annotation, '__tuple_params__') and
-              hasattr(annotation, '__tuple_use_ellipsis__')):
+              hasattr(annotation, '__tuple_use_ellipsis__')):  # for Python 3.5
             params = annotation.__tuple_params__
             if params is not None:
                 param_strings = [self.format_annotation(p) for p in params]
@@ -570,8 +619,8 @@ class Signature:
         return qualname
 
 
-def getdoc(obj, attrgetter=safe_getattr, allow_inherited=False):
-    # type: (Any, Callable, bool) -> str
+def getdoc(obj: Any, attrgetter: Callable = safe_getattr,
+           allow_inherited: bool = False) -> str:
     """Get the docstring for the object.
 
     This tries to obtain the docstring for some kind of objects additionally:
